{******************************************************************************}
{                                                                              }
{           ReportBuilder Data Access Developement Environment (DADE)          }
{                                                                              }
{                           Jason Sweby <jason@carval.co.uk>                   }
{             Copyright (c) 1999 Carval Computing Limited                      }
{             Copyright (c) 1996-1999 Digital Metaphors Corporation            }
{                                                                              }
{                                                                              }
{                                   v4.2                                       }
{******************************************************************************}

unit daDBTitan;

interface

uses Classes, SysUtils, Forms, ExtCtrls, ppClass, ppDBPipe, ppDB, ppDBBDE,
     ppClasUt, ppUtils, ppTypes, daDB, daQuery, daDataVw, daDatPrv, daQClass,
     AccMain, BSEMain, DB;

type

  {Titan Dependent DataView Classes:

     1.  Titan TDataSet descendants
           - TDataSets that can be children of a DataView.
           - Override the HasParent method of TComponent to return True
           - Must be registerd with the Delphi IDE using the RegisterNoIcon procedure

       a. TdaChildTitanQuery - TQuery descendant that can be a child of a DataView
       b. TdaChildTitanTable - TTable descendant that can be a child of a DataView
       c. TdaChildTitanStoredProc - TStoredProc descendant that can be a child of a DataView


     2.  TdaTitanSession
           - descendant of TppSession
           - implements GetDatabaseNames, GetTableNames, etc.

     3.  TdaTitanDataSet
          - descendant of TppDataSet
          - implements GetFieldNames for SQL

     4.  TdaTitanQueryDataView
          - descendant of TppQueryDataView
          - uses the above classes to create the required
            Query -> DataSource -> Pipeline -> Report connection
          - uses the SQL built by the QueryWizard to assign
            SQL to the TQuery etc.

      }

  { TdaChildTitanQuery }
  TdaChildTitanQuery = class(TtaQuery)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildTitanQuery}

  { TdaChildTitanTable }
  TdaChildTitanTable = class(TtaTable)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildTitanTable}

  { TdaChildTitanStoredProc }
  TdaChildTitanStoredProc = class(TtaStoredProc)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildTitanStoredProc}


  { TdaTitanSession }
  TdaTitanSession = class(TdaSession)
    private
      function  GetAliasDriverName(const aAlias: String): String;

    protected
      procedure SetDataOwner(aDataOwner: TComponent); override;

    public
      class function ClassDescription: String; override;
      class function DataSetClass: TdaDataSetClass; override;
      class function DatabaseClass: TComponentClass; override;

      procedure GetDatabaseNames(aList: TStrings); override;
      function GetDatabaseType(const aDatabaseName: String): TppDatabaseType; override;
      procedure GetTableNames(const aDatabaseName: String; aList: TStrings); override;
      function ValidDatabaseTypes: TppDatabaseTypes; override;

  end; {class, TdaTitanSession}


  { TdaTitanDataSet }
  TdaTitanDataSet = class(TdaDataSet)
    private
      FQuery: TtaQuery;

      function GetQuery: TtaQuery;

    protected
      procedure BuildFieldList; override;
      function  GetActive: Boolean; override;
      procedure SetActive(Value: Boolean); override;
      procedure SetDatabaseName(const aDatabaseName: String); override;
      procedure SetDataName(const aDataName: String); override;

      property Query: TtaQuery read GetQuery;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function ClassDescription: String; override;

      procedure GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings); override;
      procedure GetFieldsForSQL(aList: TList; aSQL: TStrings); override;

  end; {class, TdaTitanDataSet}


  { TdaTitanQueryDataView }
  TdaTitanQueryDataView = class(TdaQueryDataView)
    private
      FDataSource: TppChildDataSource;
      FQuery: TdaChildTitanQuery;

    protected
      procedure Loaded; override;
      procedure UpdateQueryObject;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function PreviewFormClass: TFormClass; override;
      class function SessionClass: TClass; override;

      procedure Init; override;
      procedure ConnectPipelinesToData; override;

      procedure SQLChanged; override;

    published
      property DataSource: TppChildDataSource read FDataSource;

  end; {class, TdaTitanQueryDataView}




  { TdaTitanTemplateDataView }
  TdaTitanTemplateDataView = class(TdaCustomDataView)
    private
      FDatabaseName: String;

    protected
      procedure SetDatabaseName(aDatabaseName: String); virtual;

    public
      {defines TitanSession as the session class}
      class function SessionClass: TClass; override;

      {returns True}
      class function IsTemplate: Boolean; override;

      {descendants should call these to create data access objects}
      function CreateQuery: TtaQuery;
      function CreateTable(aTableName: String): TtaTable;
      function CreateStoredProc(aStoredProcName: String): TtaStoredProc;
      function CreateDataSource: TDataSource;
      function CreateDataPipeline: TppDBPipeline;
      function CreatePipelineField(aTableName, aFieldName, aFieldAlias: String;
                             aDataPipeline: TppDataPipeline; aSearchable, aSortable: Boolean): TppField;

      property DatabaseName: String read FDatabaseName write SetDatabaseName;

    published

      property Report;

  end; {class, TdaTitanTemplateDataView}



  { TdaTitanQueryTemplateDataView }
  TdaTitanQueryTemplateDataView = class(TdaTitanTemplateDataView)
    private
      FQuery: TtaQuery;
      FDataSource: TDataSource;
      FDataPipeline: TppDBPipeline;
      FSQL: TdaSQL;

    protected
      procedure Loaded; override;
      procedure SetDatabaseName(aDatabaseName: String); override;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      procedure CreateDataPipelines; override;
      procedure CreatePipelineFields(aDataPipeline: TppDataPipeline); override;

      procedure ConnectPipelinesToData; override;
      procedure DefineDataSelection; override;

      procedure Init; override;

      function  AddSelectTable(aTableName: String): TdaTable;
      function  AddSelectField(aTable: TdaTable; aFieldName, aFieldAlias: String; aSearchable, aSortable: Boolean): TdaField;
      procedure DefineSelectedFields; virtual;
      procedure DefineCalculatedFields; virtual;
      procedure DefineSelectionCriteria; virtual;
      procedure DefineSortOrder; virtual;

      procedure BuildSQL;

      property SQL: TdaSQL read FSQL;

    published
      property DataPipeline: TppDBPipeline read FDataPipeline write FDataPipeline;
      property Query: TtaQuery read FQuery;
      property DataSource: TDataSource read FDataSource;

  end; {class, TdaTitanQueryTemplateDataView}


procedure Register;

implementation

procedure Register;
begin

  {Titan DataAccess Components}
  RegisterNoIcon([TdaChildTitanQuery, TdaChildTitanTable, TdaChildTitanStoredProc]);

  {Titan DataViews}
  RegisterNoIcon([TdaTitanQueryDataView]);


end;


{******************************************************************************
 *
 ** C H I L D   T I T A N  D A T A   A C C E S S   C O M P O N  E N T S
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaChildTitanQuery.HasParent }

function TdaChildTitanQuery.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{------------------------------------------------------------------------------}
{ TdaChildTitanTable.HasParent }

function TdaChildTitanTable.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}


{------------------------------------------------------------------------------}
{ TdaChildTitanStoredProc.HasParent }

function TdaChildTitanStoredProc.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}




{******************************************************************************
 *
 ** T I T A N   S E S S I O N
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanSession.ClassDescription }

class function TdaTitanSession.ClassDescription: String;
begin
  Result := 'TitanAccessSession';
end; {class function, ClassDescription}


{------------------------------------------------------------------------------}
{ TdaTitanSession.DataSetClass }

class function TdaTitanSession.DataSetClass: TdaDataSetClass;
begin
  Result := TdaTitanDataSet;
end; {class function, DataSetClass}

{------------------------------------------------------------------------------}
{ TdaTitanSession.DatabaseClass }

class function TdaTitanSession.DatabaseClass: TComponentClass;
begin
  Result := TttDatabase;
end; {class function, DatabaseClass}

{------------------------------------------------------------------------------}
{ TdaTitanSession.GetAliasDriverName }

function TdaTitanSession.GetAliasDriverName(const aAlias: String): String;
var
  lDatabase: TttDatabase;
begin

 { Result := 'STANDARD';
  Exit;}

  Result := '';

  {try to get driver name from the session}
  if taSession.IsAlias(aAlias) then
    Result := taSession.GetAliasDriverName(aAlias)
  else
    begin

      {try to get the TDatabase object}
      lDatabase := taSession.FindDatabase(aAlias);

      {return the drivername using Database.AliasName or Database.DriverName as appropriate}
      if lDatabase <> nil then
        if taSession.IsAlias(lDatabase.AliasName) then
          Result := taSession.GetAliasDriverName(lDatabase.AliasName)
        else
          Result := lDatabase.DatabaseName;

    end;

end; {function, GetAliasDriverName}


{------------------------------------------------------------------------------}
{ TdaTitanSession.GetTableNames }

procedure TdaTitanSession.GetTableNames(const aDatabaseName: String; aList: TStrings);
begin
  taSession.GetTableNames(aDatabaseName, '*.*', True, False, aList);
end; {procedure, GetTableNames}

{------------------------------------------------------------------------------}
{ TdaTitanSession.GetDatabaseNames }

procedure TdaTitanSession.GetDatabaseNames(aList: TStrings);
begin
  taSession.GetDataBaseNames(aList);
end; {procedure, GetDatabaseNames}

{------------------------------------------------------------------------------}
{ TdaTitanSession.GetDatabaseType }
function TdaTitanSession.GetDatabaseType(const aDatabaseName: String): TppDatabaseType; 
Begin
     Result := dtMSAccess;
End; {function, GetDatabaseType}

{------------------------------------------------------------------------------}
{ TdaTitanSession.SetDataOwner }
procedure TdaTitanSession.SetDataOwner(aDataOwner: TComponent);
Var
   lList: TStringList;
Begin
     inherited SetDataOwner(aDataOwner);

     lList := TStringList.Create;

     GetDatabaseNames(lList);

     lList.Free;
End; {procedure, SetDataOwner}

{------------------------------------------------------------------------------}
{ TdaTitanSession.ValidDatabaseTypes }
function TdaTitanSession.ValidDatabaseTypes: TppDatabaseTypes;
Begin
     Result := [dtMSAccess];
End; {function, ValidDatabaseTypes}
{******************************************************************************
 *
 ** T I T A N   D A T A S E T
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.Create }

constructor TdaTitanDataSet.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  FQuery := nil;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.Destroy }

destructor TdaTitanDataSet.Destroy;
begin

  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.ClassDescription }

class function TdaTitanDataSet.ClassDescription: String;
begin
  Result := 'TitanAccessDataSet';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.GetActive }

function TdaTitanDataSet.GetActive: Boolean;
begin
  Result := GetQuery.Active
end; {function, GetActive}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.SetActive }

procedure TdaTitanDataSet.SetActive(Value: Boolean);
begin
  GetQuery.Active := Value;
end; {procedure, SetActive}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.GetQuery }

function TdaTitanDataSet.GetQuery: TtaQuery;
begin

  if (FQuery = nil) then
    FQuery := TtaQuery.Create(Self);

  Result := FQuery;

end; {procedure, GetQuery}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.SetDatabaseName }

procedure TdaTitanDataSet.SetDatabaseName(const aDatabaseName: String);
begin

  inherited SetDatabaseName(aDatabaseName);

  if GetQuery.Active then
    FQuery.Active := False;

  FQuery.DatabaseName := aDatabaseName;

end; {procedure, SetDatabaseName}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.SetDataName }

procedure TdaTitanDataSet.SetDataName(const aDataName: String);
begin

  inherited SetDataName(aDataName);

  {dataset cannot be active to set data name}
  if GetQuery.Active then
    FQuery.Active := False;

  {construct an SQL statment that returns an empty result set,
   this is used to get the field information }
  FQuery.SQL.Text := 'SELECT * FROM ' + aDataName +
                     ' WHERE ''c'' <> ''c'' ';

end; {procedure, SetDataName}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.BuildFieldList }

procedure TdaTitanDataSet.BuildFieldList;
var
  liIndex: Integer;
  lTitanField: TField;
  lField: TppField;
begin

  inherited BuildFieldList;

  if not(GetQuery.Active) then
    try
      FQuery.Active := True;
    except
      on E:Exception do
        Exit;
    end; {try, except}

  for liIndex := 0 to FQuery.FieldCount - 1 do
    begin
      lTitanField := FQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.TableName    := DataName;
      lField.FieldName    := lTitanField.FieldName;
      lField.DataType     := ppConvertFieldType(lTitanField.DataType);

      AddField(lField);
    end;

end; {function, BuildFieldList}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.GetFieldNamesForSQL }

procedure TdaTitanDataSet.GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings);
var
  lQuery: TtaQuery;
begin
  aList.Clear;

  lQuery := TtaQuery.Create(Self);

  lQuery.DatabaseName := DatabaseName;
  lQuery.SQL := aSQL;

  lQuery.GetFieldNames(aList);

  lQuery.Free;

end; {procedure, GetFieldNamesForSQL}

{------------------------------------------------------------------------------}
{ TdaTitanDataSet.GetFieldsForSQL }

procedure TdaTitanDataSet.GetFieldsForSQL(aList: TList; aSQL: TStrings);
var
  lQuery: TtaQuery;
  lTitanField: TField;
  lField: TppField;
  liIndex: Integer;
begin
  aList.Clear;

  lQuery := TtaQuery.Create(Self);

  lQuery.DatabaseName := DatabaseName;
  lQuery.SQL := aSQL;

  lQuery.Active := True;

  for liIndex := 0 to lQuery.FieldCount - 1 do
    begin
      lTitanField := lQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.FieldName    := lTitanField.FieldName;
      lField.DataType     := ppConvertFieldType(lTitanField.DataType);

      aList.Add(lField);
    end;

  lQuery.Free;

end; {procedure, GetFieldsForSQL}


{******************************************************************************
 *
 ** T I T A N  Q U E R Y   D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.Create }

constructor TdaTitanQueryDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {notes: 1. must use ChildQuery, ChildDataSource, ChildPipeline etc.
          2. use Self as owner for Query, DataSource etc.
          3. do NOT assign a Name }

  FQuery := TdaChildTitanQuery.Create(Self);
  FDataSource := TppChildDataSource.Create(Self);

  FDataSource.DataSet := FQuery;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.Destroy }

destructor TdaTitanQueryDataView.Destroy;
begin
  FDataSource.Free;
  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.SessionClass }

class function TdaTitanQueryDataView.SessionClass: TClass;
begin
  Result := TdaTitanSession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.Loaded }

procedure TdaTitanQueryDataView.Loaded;
begin

  inherited Loaded;

  UpdateQueryObject;

end; {procedure, Loaded}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.ConnectPipelinesToData }

procedure TdaTitanQueryDataView.ConnectPipelinesToData;
begin

  if DataPipelineCount = 0 then Exit;

  {need to reconnect here}
  TppDBPipeline(DataPipelines[0]).DataSource := FDataSource;


end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.PreviewFormClass }

class function TdaTitanQueryDataView.PreviewFormClass: TFormClass;
begin
  Result := TFormClass(GetClass('TdaPreviewDataDialog'));
end; {class function, PreviewFormClass}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.Init }

procedure TdaTitanQueryDataView.Init;
var
  lDataPipeline: TppChildDBPipeline;

begin

  inherited Init;

  if DataPipelineCount > 0 then Exit;

  {note: DataView's owner must own the DataPipeline }
  lDataPipeline := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));
  lDataPipeline.DataSource := FDataSource;

  lDataPipeline.AutoCreateFields := False;

  {add DataPipeline to the dataview }
  lDataPipeline.DataView := Self;

end; {procedure, Init}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.SQLChanged }

procedure TdaTitanQueryDataView.SQLChanged;
begin
  UpdateQueryObject;
end; {procedure, SQLChanged}

{------------------------------------------------------------------------------}
{ TdaTitanQueryDataView.UpdateQueryObject }

procedure TdaTitanQueryDataView.UpdateQueryObject;
begin

  FQuery.DatabaseName := SQL.DatabaseName;
  FQuery.SQL := SQL.SQLText;

end; {procedure, UpdateQueryObject}



{******************************************************************************
 *
 ** T I T A N  T E M P L A T E  D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.IsTemplate }

class function TdaTitanTemplateDataView.IsTemplate: Boolean;
begin
  Result := True;
end; {function, IsTemplate}



{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.SessionClass }

class function TdaTitanTemplateDataView.SessionClass: TClass;
begin
  Result := TdaTitanSession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.SetDatabaseName }

procedure TdaTitanTemplateDataView.SetDatabaseName(aDatabaseName: String);
begin
  FDatabaseName := aDatabaseName;

end; {procedure, SetDatabaseName}


{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.CreateQuery }

function TdaTitanTemplateDataView.CreateQuery: TtaQuery;
begin
  Result := TdaChildTitanQuery.Create(Self);
  Result.DatabaseName := DatabaseName;

end; {procedure, CreateQuery}

{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.CreateTable }

function TdaTitanTemplateDataView.CreateTable(aTableName: String): TtaTable;
begin
  Result := TdaChildTitanTable.Create(Self);
  Result.DatabaseName := DatabaseName;
  Result.TableName := aTableName;


end; {procedure, CreateTable}

{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.CreateStoredProc }

function TdaTitanTemplateDataView.CreateStoredProc(aStoredProcName: String): TtaStoredProc;
begin
  Result := TdaChildTitanStoredProc.Create(Self);
  Result.DatabaseName := DatabaseName;
  Result.StoredProcName := aStoredProcName;


end; {procedure, CreateStoredProc}

{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.CreateDataSource }

function TdaTitanTemplateDataView.CreateDataSource: TDataSource;
begin
  Result := TppChildDataSource.Create(Self);

end; {procedure, CreateDataSource}

{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.CreateDataPipeline }

function TdaTitanTemplateDataView.CreateDataPipeline: TppDBPipeline;
begin

  {note: DataView's owner must own the DataPipeline }
  Result := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));

  {add pipeline to the dataview's Pipelines[] array}
  Result.DataView := Self;

end; {procedure, CreateDataPipeline}



{------------------------------------------------------------------------------}
{ TdaTitanTemplateDataView.CreatePipelineField }

function TdaTitanTemplateDataView.CreatePipelineField(aTableName, aFieldName, aFieldAlias: String;
                             aDataPipeline: TppDataPipeline; aSearchable, aSortable: Boolean): TppField;
begin

  Result := TppField.Create(nil);

  {set field props}
  Result.TableName     := aTableName;
  Result.FieldName     := aFieldName;

  if (aFieldAlias <> '') then
    Result.FieldAlias    := aFieldAlias
  else
    Result.FieldAlias    := aFieldName;

  Result.DataType      := aDataPipeline.GetFieldDataType(aFieldName);
  Result.FieldLength   := aDataPipeline.GetFieldSize(aFieldName);
  Result.DisplayWidth  := aDataPipeline.GetFieldDisplayWidth(aFieldName);


  Result.Searchable    := aSearchable;
  Result.Sortable      := aSortable;

  {add field to the datapipeline's Fields[] array}
  Result.DataPipeline := aDataPipeline;

  {turn auto create fields off}
  if (aDataPipeline <> nil) and (aDataPipeline is TppDBPipeline) and
    TppDBPIpeline(aDataPipeline).AutoCreateFields then
    TppDBPIpeline(aDataPipeline).AutoCreateFields := False;


end; {procedure, CreatePipelineField}




{******************************************************************************
 *
 ** Query Template DataView
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.Create }

constructor TdaTitanQueryTemplateDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {call CreateQuery method defined in the ancestor }
  FQuery := CreateQuery;

  FSQL := TdaSQL.Create(Self);
  FSQL.Session := TdaSessionClass(SessionClass).Create(Self);

  {call CreateDataSource method defined in the ancestor }
  FDataSource := CreateDataSource;
  FDataSource.DataSet := FQuery;

  FDataPipeline := nil;

  AskAtRuntime := True; {This was commented out by default - JS}

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.Destroy }

destructor TdaTitanQueryTemplateDataView.Destroy;
begin

  inherited Destroy;

end; {destructor, Destroy}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.Loaded }


procedure TdaTitanQueryTemplateDataView.Loaded;
begin

  inherited Loaded;

  DefineSelectedFields;
  DefineDataSelection;

end; {procedure, Loaded}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.SetDatabaseName }

procedure TdaTitanQueryTemplateDataView.SetDatabaseName(aDatabaseName: String);
begin
  inherited SetDatabaseName(aDatabaseName);

  FQuery.DatabaseName := DatabaseName;
  FSQL.DatabaseName   := DatabaseName;

end; {procedure, SetDatabaseName}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.Init }


procedure TdaTitanQueryTemplateDataView.Init;
begin

  if (FDataPipeline <> nil) then Exit;

  DefineSelectedFields;
  DefineDataSelection;

  inherited Init;


end; {procedure, Init}



{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.CreateDataPipelines }

procedure TdaTitanQueryTemplateDataView.CreateDataPipelines;
begin

  {note: call CreateDataPipeline defined in ancestor }
  FDataPipeline := CreateDataPipeline;
  FDataPipeline.UserName := 'pl' + ppTextToIdentifier(ClassDescription);

end; {procedure, CreateDataPipelines}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.CreatePipelineFields }

procedure TdaTitanQueryTemplateDataView.CreatePipelineFields(aDataPipeline: TppDataPipeline);
var
  liIndex: Integer;

begin

  if (FDataPipeline = aDataPipeline) then
    begin

      TppDBPipeline(aDataPipeline).AutoCreateFields := False;

      for liIndex := 0 to FSQL.SelectFieldCount-1 do
        begin
          with FSQL.SelectFields[liIndex] do
            CreatePipelineField(TableAlias, FieldName, FieldAlias, aDataPipeline, Searchable, Sortable);

        end;

    end;

end; {procedure, CreatePipelineFields}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.ConnectPipelinesToData }

procedure TdaTitanQueryTemplateDataView.ConnectPipelinesToData;
begin

  if (FDatapipeline = nil) then Exit;

  {connect datapipeline to datasource here}
  FDatapipeline.DataSource := FDataSource;

end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.DefineSelectedFields }

procedure TdaTitanQueryTemplateDataView.DefineSelectedFields;
begin

  FSQL.Clear;

 {descendants add code here}

end; {procedure, DefineSelectedFields}

{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.DefineCalculatedFields }

procedure TdaTitanQueryTemplateDataView.DefineCalculatedFields;
begin

 {descendants add code here}

end; {procedure, DefineCalculatedFields}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.AddSelectTable }

function TdaTitanQueryTemplateDataView.AddSelectTable(aTableName: String): TdaTable;
begin
  Result := FSQL.AddTable(aTableName);

end; {function, AddSelectTable}

{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.AddSelectField }

function TdaTitanQueryTemplateDataView.AddSelectField(aTable: TdaTable; aFieldName, aFieldAlias: String; aSearchable, aSortable: Boolean): TdaField;
begin


  Result := FSQL.AddSelectField(aTable, aFieldName);

  if (aFieldAlias <> '')  then
    Result.FieldAlias := aFieldAlias;

  Result.Searchable := aSearchable;
  Result.Sortable   := aSortable;

end; {procedure, AddSelectField}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.DefineDataSelection }

procedure TdaTitanQueryTemplateDataView.DefineDataSelection;
begin

  SetActive(False);

  BuildSQL;

end; {procedure, DefineDataSelection}

{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.BuildSQL }

procedure TdaTitanQueryTemplateDataView.BuildSQL;
begin

  DefineSelectionCriteria;

  DefineSortOrder;

  FQuery.SQL := FSQL.SQLText;

end; {procedure, BuildSQL}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.DefineSelectionCriteria }

procedure TdaTitanQueryTemplateDataView.DefineSelectionCriteria;
var
  liIndex: Integer;
  lField: TppField;
  lTable: TdaTable;

begin

  if (FDataPipeline = nil) then Exit;


  {add code for selection criteria}
  FSQL.ClearCriteria;

  for liIndex := 0 to FDataPipeline.FieldCount-1 do
    if FDataPipeline.Fields[liIndex].Search then
      begin
        lField := FDataPipeline.Fields[liIndex];
        lTable := FSQL.GetTableForSQLAlias(lField.TableName);

        FSQL.AddCriteriaField(lTable, lField.FieldName, dacoLike, lField.SearchExpression);


      end;

end; {procedure, DefineSelectionCriteria}


{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.DefineSortOrder }

procedure TdaTitanQueryTemplateDataView.DefineSortOrder;

var
  liIndex: Integer;
  lField: TppField;
  ldaField: TdaField;
  lTable: TdaTable;
  lbAscending: Boolean;

begin

  if (FDataPipeline = nil) then Exit;

  {add code for sort order}
  FSQL.ClearOrderByFields;


  for liIndex := 0 to FDataPipeline.FieldCount-1 do
    if FDataPipeline.Fields[liIndex].Sort then
      begin

        lField := FDataPipeline.Fields[liIndex];

        lTable := FSQL.GetTableForSQLAlias(lField.TableName);

        ldaField := FSQL.GetFieldForTable(lTable, lField.FieldName);

        lbAscending := (lField.SortType = soAscending);

        FSQL.AddOrderByField(ldaField, lbAscending);

      end;


end; {procedure, DefineSortOrder}

{******************************************************************************
 *
 ** I N I T I A L I Z A T I O N   /   F I N A L I Z A T I O N
 *
{******************************************************************************}

initialization

  RegisterClasses([TdaChildTitanQuery, TdaChildTitanTable, TdaChildTitanStoredProc]);

  daRegisterSession(TdaTitanSession);
  daRegisterDataSet(TdaTitanDataSet);

  daRegisterDataView(TdaTitanQueryDataView);

finalization

  UnRegisterClasses([TdaChildTitanQuery, TdaChildTitanTable, TdaChildTitanStoredProc]);

  daUnRegisterSession(TdaTitanSession);
  daUnRegisterDataSet(TdaTitanDataSet);

  daUnRegisterDataView(TdaTitanQueryDataView);

end.
